import {App, inject, Plugin, PropType} from "vue"
import {$vfm, vfmPlugin, VueFinalModalProperty} from "vue-final-modal"

/**
 * VueFinalModal 을 사용하기 위한 옵션
 */
export const modalOption = {
  key: "$modal",
  componentName: "Modal",
  dynamicContainerName: "ModalsContainer"
}

/**
 * VueFinalModal Plugin
 * ```
 * createApp(Main)
 *     .use(ModalPlugin)
 *     .mount('#app')
 * ```
 */
export const ModalPlugin: Plugin & {
  modal: VueFinalModalProperty
} = {

  install: (app: App): void => {
    vfmPlugin().install(app, modalOption)
  },

  // $vfm object in vue-final-modal
  modal: $vfm
}

export type ModalClose = () => void

export type ModalProps = {
  width?: string | number
  height?: string | number
  minWidth?: string | number
  minHeight?: string | number
  maxWidth?: string | number
  maxHeight?: string | number
  drag?: boolean
  clickToClose?: boolean
  escToClose?: boolean
  transition?: string
  classes?: string
}

export const modalProps = {
  type: Object as PropType<ModalProps>,
  default: (): ModalProps => ({})
}

export const useModal = (): VueFinalModalProperty => inject('$modal') as VueFinalModalProperty